/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/basic.js TAP single stream pipeline just wraps > got expected data 1`] = `
Array [
  "written to pipeline",
  "emitted by 1",
  "emitted by 2",
  "emitted by 3",
  "ending pipeline",
]
`

exports[`test/basic.js TAP wrap some streams > got expected data 1`] = `
Array [
  "written to pipeline",
  "emitted by 1",
  "emitted by 2",
  "emitted by 3",
  "ending pipeline",
]
`
